#!/usr/local/bin/perl5 -w

# This script reads the gamelist file and runs the games specified

use File::Copy;

$usage = "$0";

###

sub catch_zap {
  $exit = 1;
}

$SIG{INT} = \&catch_zap;

###

use vars qw($usage);
$usage = "run_game_queue.pl\n";

use File::Basename;
$dir = dirname($0); #gets the directory of this script

require "$dir/support.pl";

($qdir, undef, undef, $gamelistfn, undef, $rungamelockfn, $queueemptyfile) = &lock_info;
$gamelisttmpfn = "$qdir/gamelist.$$.tmp";

$hostname = `hostname`;
chomp $hostname;

$exit = 0;
$games_played = 0;

if (-e $rungamelockfn) {
  if (defined($global_notify_addr)) {
    mail_notify($global_notify_addr, "$hostname", "$date: Tried to run when another run_game_queue.pl was running!");
  }
  die "It seems a run_game_queue.pl script is already running. If not, please remove $rungamelockfn";
}

open (LOCK, ">$rungamelockfn") || die "Could not open run game lock file: $!";
print LOCK $$;
close LOCK;

# Make a copy of the game queue file
$gamelistsavefn = "$qdir/gamelist.rungamesave";
copy $gamelistfn, $gamelistsavefn || die "Could not copy gamelist file from '$gamelistfn' ro '$gamelistsavefn': $!";

while (!$exit) {
  (acquire_lock(-1, "run_game_queue.pl")==0) || die "Could not acquire queue lock!";

  open (GAMELIST, "<$gamelistfn") || die "Could not open game list file: $!";
  open (GAMELISTTMP, ">$gamelisttmpfn") || die "Could not open game list tmp file: $!";

  $haveline = 0;
  while ($_ = <GAMELIST>) {
    if ($haveline || /^\s*\#/ || /^\s*$/) {
      print GAMELISTTMP $_;
      next;
    }
    #we have a real line to use! First lets get the rest of the file to the tmp file
    $haveline = 1;
    $line = $_;
  }

  close GAMELIST;
  close GAMELISTTMP;
  unlink $gamelistfn or die "Could not unlink game list file: $!";
  rename $gamelisttmpfn, $gamelistfn or die "Could not rename tmp file: $!";
  &release_lock;

  if ($haveline) {
    unlink $queueemptyfile;
    if ($line =~ /^\s*exit\s*/) {
      #zephyr_notify($global_notify_zaddr, "$hostname: I got an exit in my queue") if (defined($global_notify_zaddr));
      $exit = 1;
    } else {
      chomp $line;
      ($date, $slow_fac, $conf, $team1, $team1arg, $team2, $team2arg) = split /\s+/, $line;
      print "About to run: $line\n";
      $currdate = `date`;
      chomp $currdate;
      $cmdline = "$dir/runtrial $slow_fac $conf $team1 $team1arg $team2 $team2arg";
      print "$currdate: Command to be run (queued on $date):\n $cmdline\n";
      system "$cmdline";
      #sleep 4;

      $games_played++;
    }
  } else {
    $date = `date`;
    chomp $date;
    if (defined($global_notify_addr) && $games_played > 0) {
      mail_notify($global_notify_addr, "$hostname", "$date: My queue is empty");
    }
    system "touch $queueemptyfile";
    $exit = 1;
  }
}


unlink $rungamelockfn;

print "Games played: $games_played\n";
if (defined($global_notify_addr) && $games_played > 0) {
  mail_notify($global_notify_addr, "$hostname", "$date: I played $games_played games");
}
