# This file is meant to be included in other perl scripts

use File::Basename;

# include the script dir in the @INC search path
use lib (dirname($0));

use Mail::Send;

### Load the global config file
$runconffn = dirname($0)."/run.conf";
require $runconffn;

### Lock stuff


#return $queuedir, $lockfile, $holderfile, $gamelist, $forbidfile, $queueemptyfile
sub lock_info {
  return ($global_queue_dir,
	  "$global_queue_dir/lock",
	  "$global_queue_dir/lock/holder",
	  "$global_queue_dir/gamelist",
	  "$global_queue_dir/forbid_run",
	  "$global_queue_dir/rungamelock",
          "$global_queue_dir/QUEUE_EMPTY");
}

# returns: 0=success 1=retry count exceeded -1=other error
sub acquire_lock {
  my ($retry_count, $usage_message) = @_;
  my ($qdir, $lockdir, $holderfile) = &lock_info;
  my ($i, $date);
  my ($ERR_EEXIST) = (17);
  for ($i = 0; $retry_count < 0 || $i < $retry_count; $i++) {
    if (mkdir $lockdir, 0777) {
      if (!open (FHUSER, ">$holderfile"))
	{
	  warn "acquire lock: could not create holder file '$holderfile': $!";
	  return -1;
	}
      $date = `date`;
      chomp $date;
      print FHUSER "$date: $usage_message";
      close FHUSER;
      return 0;
    } else {
      print "Error: ".($!+0)."\n";
      if ($! != $ERR_EEXIST)
	{
	  warn "acquire lock: mkdir '$lockdir' failed with weird error: $!";
	  return -1;
	}
      print "Waiting to acquire lock, currently held by: ".`cat $holderfile`."\n";
      sleep 1;
    }
  }
  return 1;
}

#returns 0=success 1=lock not held -1=other error
sub release_lock {
  my ($qdir, $lockdir, $holderfile) = &lock_info;
  if (not -e $lockdir) {
    warn "Tried to release lock when it was not held";
    return 1;
  }
  unlink "$holderfile" || warn "Could not unlink holder file";
  if (not rmdir "$lockdir") {
    warn "Could not release lock: $!";
    return -1;
  }
  return 0;
}

sub get_lock_holder {
  my ($qdir, $lockdir, $holderfile) = &lock_info;
  my (@file);
  return "LOCK NOT HELD" if (not -e $lockdir);
  open (FHHOLDER, "<$holderfile") || warn "Could not open holder file: $!";
  @file = <FHHOLDER>;
  close FHHOLDER;
  return @file;
}

### notification stuff
sub zephyr_notify {
  my ($addr, $mess) = @_;
  if (not defined($addr)) {
    warn "Told to send zephyr to undefined addr";
    return;
  }
  open (ZWPROC, "|zwrite -i AUTOGAMES $addr") || warn "Could not open zwrite process: $!";
  print ZWPROC $mess;
  close ZWPROC;
}

require Mail::Send;

sub mail_notify {
  my ($addr, $subject, $mess) = @_;
  my ($msg, $fh);
  $msg = new Mail::Send;
  $msg->to($addr);
  $msg->subject("[AUTOGAMES] $subject");
  $fh = $msg->open;
  print $fh $mess;
  $fh->close;         # complete the message and send it
}
